# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CPUtils.Extensions.utils import sendmail

#def notifyManagerByMail(self, state_change, **kw):
    #""" WF script used for ploneboard_conversation_workflow """
    #object = state_change['object']

    #send_from_address_param = "support@communesplone.be"
    #send_to_address_param = "support@communesplone.be"
    #subject_param = "Nouvelle conversation '%s'" % object.Title()
    #comment_param = "Une nouvelle conversation a été ajoutée à %s" % object.aboslute_url()

    #try:
        #plone_tool = getToolByName(self, 'plone_utils')
        #host = plone_tool.getMailHost()
        #encoding = plone_tool.getSiteEncoding()
        #message = comment_param
        #send_to_address = send_to_address_param
        #send_from_address = send_from_address_param
        #subject = subject_param
        #return host.secureSend(message, send_to_address,
                               #send_from_address, subject=subject,
                               #subtype='plain', charset=encoding,
                               #debug=False, From=send_from_address)
    #except Exception, message:
        #print "There was a problem during the send of an e-mail for object : %s." % object
        #print "The exception is : %s." % message

    #return

def notifyConvMembersByMail(self, state_change, **kw):
    """
        This script is used to send mail to :
            * all persons participating to a conversation
            * a list of fixed emails (optional)
        To activate the functionality :
            1) go in the ZMI to "portal_workflow/ploneboard_comment_workflow/scripts"
            2) add an "External Method"
            3) complete the following information :
                . id = "notifyConvMembersByMail" or whatever you want
                . title = "notifyConvMembersByMail" or whatever you want
                . module name = "CPUtils.ploneboard_wf_scripts"
                . function name = "notifyConvMembersByMail"
            4) go in the ZMI to the appropriate transition (by example autopublish)
            5) select for an empty field "Script (before)" or "Script (after)" your External Method id
            6) add a property in portal_ploneboard with the following information
                . name = "auto_send_to_addresses"
                . type = "string"
                . value = "test@x.be test2@x.be" all addresses separated by spaces
    """
    object = state_change['object']
    portal = getToolByName(self, 'portal_url').getPortalObject()

    #get the container, the PloneboardConversation
    container = object.getParentNode()

    #we will build a list of e-mails to send to
    emails=[]
    if portal.portal_ploneboard.hasProperty('auto_send_to_addresses'):
        emails = portal.portal_ploneboard.auto_send_to_addresses.strip().split()

    plone_tool = getToolByName(self, 'plone_utils')
    mt = portal.portal_membership

    #loop on every comments...
    comments = portal.portal_catalog.searchResults(portal_type="PloneboardComment", path="/".join(container.getPhysicalPath()))
    i = 0
    for comment in comments:
        i = i + 1
        creator = comment.Creator
        user = mt.getMemberById(creator)
        if user is not None:
            email = user.getProperty('email')
            if email not in emails:
                emails.append(email)

    #we remove the e-mail of the current comment creator as it is not necessary for him to be warned
    creator = object.Creator()
    user = mt.getMemberById(creator)
    email = user.getProperty('email')
    emails.remove(email)

    send_from_address_param = portal.email_from_address
    send_to_address_param = emails
    if i == 1:
        #a new conversation has been started and contain only one comment
        subject_param = "Nouvelle conversation '%s'" % container.Title()
        comment_param = "Une nouvelle conversation '%s' a démarré à %s." % (container.Title(), container.absolute_url())
    else:
        subject_param = "Nouveau commentaire pour la conversation '%s'" % container.Title()
        comment_param = "Un nouveau commentaire a été ajouté à la conversation '%s' à %s." % (container.Title(), container.absolute_url())

    try:
        host = plone_tool.getMailHost()
        encoding = plone_tool.getSiteEncoding()
        message = comment_param
        send_to_address = ''
        mbcc = ','.join(send_to_address_param)
        send_from_address = send_from_address_param
        subject = subject_param
        #MUST BE REPLACED AS FOLLOWING METHOD
        result = host.secureSend(message, send_to_address,
                        send_from_address, mbcc=mbcc, subject=subject,
                        subtype='plain', charset=encoding,
                        debug=False, From=send_from_address)
        #print result
    except Exception, message:
        print "There was a problem during the send of an e-mail for object : %s." % object
        print "The exception is : %s." % message

    return

def notifyConvMembersByMailRIC(self, state_change, **kw):
    """
        This script is used to send mail to :
            * all persons participating to a conversation
            * a list of fixed emails (optional)
        To activate the functionality :
            1) go in the ZMI to "portal_workflow/ploneboard_comment_workflow/scripts"
            2) add an "External Method"
            3) complete the following information :
                . id = "notifyConvMembersByMail" or whatever you want
                . title = "notifyConvMembersByMail" or whatever you want
                . module name = "CPUtils.ploneboard_wf_scripts"
                . function name = "notifyConvMembersByMailRIC"
            4) go in the ZMI to the appropriate transition (by example autopublish)
            5) select for an empty field "Script (before)" or "Script (after)" your External Method id
            6) add a property in portal_ploneboard with the following information
                . name = "auto_send_to_addresses"
                . type = "string"
                . value = "test@x.be test2@x.be" all addresses separated by spaces
    """
#    return  # WHEN REPLACING WORKFLOW
    object = state_change['object']
    portal = getToolByName(self, 'portal_url').getPortalObject()

    #get the container, the PloneboardConversation
    container = object.getParentNode()

    #we will build a list of e-mails to send to
    emails=[]
    if portal.portal_ploneboard.hasProperty('auto_send_to_addresses'):
        emails = portal.portal_ploneboard.auto_send_to_addresses.strip().split()

    plone_tool = getToolByName(self, 'plone_utils')
    mt = portal.portal_membership

    #loop on every comments...
    comments = portal.portal_catalog.searchResults(portal_type="PloneboardComment", path="/".join(container.getPhysicalPath()))
    i = 0
    for comment in comments:
        i = i + 1
        creator = comment.Creator
        user = mt.getMemberById(creator)
        if user is not None:
            email = user.getProperty('email')
            if email not in emails:
                emails.append(email)

    #we remove the e-mail of the current comment creator as it is not necessary for him to be warned
    creator = object.Creator()
    user = mt.getMemberById(creator)
    email = user.getProperty('email')
    if email in emails:
        emails.remove(email)

    send_to_address_param = emails
    if i == 1:
        #a new conversation has been started and contain only one comment
        subject_param = "Nouvelle conversation '%s'" % container.Title()
        comment_param = "Une nouvelle conversation '%s' a démarré à %s." % (container.Title(), container.absolute_url())
    else:
        subject_param = "Nouveau commentaire pour la conversation '%s'" % container.Title()
        comment_param = "Un nouveau commentaire a été ajouté à la conversation '%s' à %s." % (container.Title(), container.absolute_url())

    result = sendmail(self, mfrom=portal.email_from_address, to='', body=comment_param, bcc=','.join(emails), subject=subject_param)

    #object.manage_delLocalRoles(creator)
    object.manage_addLocalRoles(creator, ('Owner','Editor',))
    return


def setCommentLocalRoles(self, state_change, **kw):
    """
        This script is used to set local roles on a comment, permitting only a comment owner to modify his comment.
        To activate the functionality :
            1) go in the ZMI to "portal_workflow/ploneboard_comment_workflow/scripts"
            2) add an "External Method"
            3) complete the following information :
                . id = "setCommentLocalRoles" or whatever you want
                . title = "setCommentLocalRoles" or whatever you want
                . module name = "CPUtils.ploneboard_wf_scripts"
                . function name = "setCommentLocalRoles"
            4) go in the ZMI to the appropriate transition (by example autopublish)
            5) select for an empty field "Script (before)" or "Script (after)" your External Method id
    """
    object = state_change['object']
    #object.manage_delLocalRoles(creator)
    object.manage_addLocalRoles(object.Creator(), ('Owner','Editor',))
    return



# -*- coding: utf-8 -*-

import os
import collections
from functools import partial


def verbose(*messages):
    print '>>', ' '.join(messages)


def error(*messages):
#    print >>sys.stderr, '!!', (' '.join(messages))
    print '!!', (' '.join(messages))


#------------------------------------------------------------------------------

def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    os.system(cmd + ' >_cmd_pv.out 2>_cmd_pv.err')
    stdout = []
    stderr = []

    def get_output(osfile, result):
        try:
            if os.path.exists(osfile):
                ofile = open(osfile, 'r')
                result += ofile.readlines()
                ofile.close()
                os.remove(osfile)
            else:
                error("File %s does not exist" % osfile)
        except IOError:
            error("Cannot open %s file" % osfile)

    get_output('_cmd_pv.out', stdout)
    get_output('_cmd_pv.err', stderr)
    return(stdout, stderr)

#------------------------------------------------------------------------------


def writeTo(filepath, data, replace=True):
    """
        Write the data in the file
    """
    if os.path.exists(filepath) and not replace:
        return verbose("%s already exists" % filepath)
    ofile = open(filepath, 'w')
    if isinstance(data, list):
        for line in data:
            ofile.write("%s\n" % (isinstance(line, unicode) and line.encode('utf8') or line))
    elif isinstance(data, str):
        ofile.write(data)
    elif isinstance(data, unicode):
        ofile.write(data.encode('utf8'))
    ofile.close()

#------------------------------------------------------------------------------


def encodeData(data, encoding='utf8'):
    """
        Encode any data to the specified encoding
    """
    if isinstance(data, unicode):
        return data.encode(encoding)
    elif isinstance(data, collections.Mapping):
        mapfunc = partial(encodeData, encoding=encoding)
        return dict(map(mapfunc, data.iteritems()))
    elif isinstance(data, collections.Iterable):
        mapfunc = partial(encodeData, encoding=encoding)
        return type(data)(map(mapfunc, data))
    else:
        return data



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Script to create and update a repository on zope instances
# Stéphan Geulette <stephan.geulette@uvcw.be>, UVCW
# From original script "recover.py" ()
#

import os, sys
from datetime import datetime
from zope.component import getSiteManager, getUtilitiesFor
from OFS.Application import Application
import socket
from utils import *

buildout_inst_type = None #True for buildout, False for manual instance
tempdir = ''
now = datetime(1973,02,12).now()
pfolders = {}
temp_added = False

###############################################################################

def main():
    global instdir, tempdir, buildout_inst_type, user, pwd
    try:
        from optparse import OptionParser
        parser = OptionParser()
        parser.add_option("-i", "--infos", dest="infos",
                      default=None,
                      help="infos about instance formatted like: "
                           "\"instance_path;transactions_days_number;admin_user;admin_password\"")
        (options, args) = parser.parse_args()
        if options.infos.startswith('#'):
            sys.exit(0)
        instdir, days, user, pwd = options.infos.split(';')
    except ValueError:
        error("Problem in parameters")
        parser.print_help()
        sys.exit(1)

    instdir = instdir.rstrip('/')
    verbose("Working on instance %s" % instdir)

    # Finding the instance type (buildout or manual)
    if os.path.exists(os.path.join(instdir,'parts')):
        buildout_inst_type = True
        #verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(instdir,'etc')):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(instdir) or True:
        error("! Invalid instance path '%s' or instance type not detected"%instdir)
        sys.exit(1)

    if buildout_inst_type:
        inst_type = 'buildout'
        zodbfilename = os.path.join(instdir, 'parts/instance/etc/zope.conf')
        fspath = os.path.join(instdir, 'var/filestorage/')
    else:
        inst_type = 'manual'
        zodbfilename = os.path.join(instdir, 'etc/zope.conf')
        fspath = os.path.join(instdir, 'var/')

    instance = os.path.basename(instdir)
    if not tempdir:
        tempdir = os.path.join(instdir, 'temp')
    hostname = socket.gethostname()

    port = treat_zopeconflines(zodbfilename, fspath)

    trace("host='%s', inst='%s'"%(hostname, instance))

    CreateAndCallExternalMethod(port,user,pwd,'cputils_checkPOSKey','utils.py','checkPOSKey')

#------------------------------------------------------------------------------

def _checkAttributes(obj, errors, context=None):
    from ZODB.POSException import POSKeyError
    # very dumb checks for list and dict (like) attributes
    # is very slow but ensures that all attributes are checked
    for k,v in obj.__dict__.items():
        if hasattr(v, 'values') and hasattr(v, 'keys'):
            try:
                data = [repr(val) for val in v.values()]
                data = [repr(val) for val in v.keys()]
            except POSKeyError, ex:
                if hasattr(obj, 'getPhysicalPath'):
                    path= '/'.join(obj.getPhysicalPath())
                elif context:
                    path = "Utility: %s in %s"%(repr(obj), '/'.join(context.getPhysicalPath()))
                else:
                    path = "Utility: %s"%(repr(obj))
                    
                error('Error %s on DICT-LIKE attribute %s (%s)' \
                     % (str(ex), k, path))
                errors.append('Error %s on DICT-LIKE attribute %s (%s)' \
                     % (str(ex), k, path))

        if hasattr(v, 'append'):
            try:
                data = [val for val in v]
            except POSKeyError, ex:
                error('Error %s on LIST-LIKE attribute %s (%s)' \
                     % (str(ex), k, '/'.join(obj.getPhysicalPath())))
                errors.append('Error %s on LIST-LIKE attribute %s (%s)' \
                     % (str(ex), k, '/'.join(obj.getPhysicalPath())))

#------------------------------------------------------------------------------

def _sub(master, errors):
    from ZODB.POSException import POSKeyError

    def check_utilities(sm, master, errors):
        for one in sm.utilities._subscribers:
            for interface in one.keys():
                try:
                    for util_tup in getUtilitiesFor(interface, context=master):
                        utility = util_tup[1]
                        _checkAttributes(utility, errors, context=master)
                except TypeError, msg:
                    error("Cannot get utilities for interface '%s' : %s"%(str(interface), msg))
        

    # check site utilities
    if master.meta_type == 'Plone Site':
        from Products.CMFCore.PortalObject import PortalObjectBase
        sm = PortalObjectBase.getSiteManager(master)
        check_utilities(sm, master, errors)

    if master.__class__ == Application:
        sm = getSiteManager()
        check_utilities(sm, master, errors)

    for oid in master.objectIds():
        try:
            obj = getattr(master, oid)
            trace('%s->%s' % ('/'.join(master.getPhysicalPath()), obj.getId()))
            #output.append('%s->%s' % ('/'.join(master.getPhysicalPath()), obj.getId()))
            if hasattr(obj, 'objectIds') and obj.getId() != 'Control_Panel':
                _sub(obj, errors)

           # check catalog explicitly
            if obj.meta_type in ['ZCatalog', 'Catalog', 'Plone Catalog Tool'] \
                or hasattr(obj, '_catalog'):
                    for idxid in obj._catalog.indexes.keys():
                        try:
                            index = obj._catalog.indexes.get(idxid)
                            trace('%s->INDEX: %s' % ('/'.join(obj.getPhysicalPath()), idxid))
                            #output.append('%s->INDEX: %s' % ('/'.join(obj.getPhysicalPath()), idxid))

                            _checkAttributes(index, errors)

                        except POSKeyError, ex:
                            error('Error %s on INDEX %s (%s)' % (str(ex), idxid, '/'.join(obj.getPhysicalPath())))
                            errors.append('Error %s on INDEX %s (%s)' % (str(ex), idxid, '/'.join(obj.getPhysicalPath())))

                    # support for lexicon
                    for lexid in obj.objectIds():
                        _checkAttributes(getattr(obj, lexid), errors)

        except POSKeyError, ex:
            error('Error %s on %s (%s)' % (str(ex), oid, '/'.join(master.getPhysicalPath())))
            errors.append('Error %s on %s (%s)' % (str(ex), oid, '/'.join(master.getPhysicalPath())))

#------------------------------------------------------------------------------

def check(app, errors=[]):
    #verbose("Begin of POSKey check")
    sys.setrecursionlimit(20000)
    _sub(app, errors)
    #verbose("End of POSKey check")

#------------------------------------------------------------------------------


if __name__ == '__main__':
    #verbose("Begin of %s"%sys.argv[0])
    main()
    #verbose("End of %s"%sys.argv[0])



#!/usr/bin/python
#
"""
    Test of zope fs file.
"""

import sys, urllib, os
import datetime
import shutil
from datetime import datetime 

buildout_inst_type = None #True for buildout, False for manual instance
FSTEST = True
FSREFS = False
PYTHONBIN = ''

def verbose(*messages):
    print '>>', ' '.join(messages)
def error(*messages):
#    print >>sys.stderr, '!!', (' '.join(messages))
    print '!!', (' '.join(messages))

#------------------------------------------------------------------------------

def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    ret = os.system(cmd + ' >_cmd_zr.out 2>_cmd_zr.err')
    stdout = stderr = []
    try:
        if os.path.exists('_cmd_zr.out'):
            ofile = open( '_cmd_zr.out', 'r')
            stdout = ofile.readlines()
            ofile.close()
            os.remove('_cmd_zr.out')
        else:
            error("File %s does not exist" % '_cmd_zr.out')
    except IOError:
        error("Cannot open %s file" % '_cmd_zr.out')
    try:
        if os.path.exists('_cmd_zr.err'):
            ifile = open( '_cmd_zr.err', 'r')
            stderr = ifile.readlines()
            ifile.close()
            os.remove('_cmd_zr.err')
        else:
            error("File %s does not exist" % '_cmd_zr.err')
    except IOError:
        error("Cannot open %s file" % '_cmd_zr.err')
    return( stdout,stderr )

#------------------------------------------------------------------------------

def read_zopeconffile(zodbfilename, lines):
    """ read the zope conf filename and include subfile """
    try:
        zfile = open( zodbfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % zodbfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('%include'):
            otherfilename = line.split()[1]
            read_zopeconffile(otherfilename, lines)
            continue
        lines.append(line)

#------------------------------------------------------------------------------

def treat_zopeconflines(zodbfilename):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)

    port = ''
    httpflag = False
    fsflag = False
    dbs = []
    dbname = ''
    for line in lines:
        #verbose("=>'%s'"%line)
        if line.startswith('<http-server>'):
            httpflag = True
            continue
        if httpflag and line.startswith('address'):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith('<zodb_db'):
            if dbname:
                error("\tnext db found while fs not found: previous dbname '%s', current line '%s'"%(dbname, line))
            dbname = line.split()[1]
            dbname = dbname.rstrip('>')
            if dbname == 'temporary':
                dbname = ''
        if line.startswith('<filestorage>'):
            fsflag = True
            continue
        if fsflag and line.startswith('path'):
            fsflag = False
            fs = line.split()[1]
            fs = os.path.basename(fs)
            dbs.append([dbname, fs])
            dbname = ''
            continue
    verbose("\tport='%s', dbs='%s'"%(port, ';'.join([','.join(dbinfo) for dbinfo in dbs])))

    if not port:
        error("! the port was not found in the config file '%s'"%zodbfilename)

    return(port, dbs)

#------------------------------------------------------------------------------

def read_zopectlfile(zopectlfilename):
    """ read the zopectl file to find the zope path """
    try:
        zfile = open( zopectlfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % zopectlfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('ZOPE_HOME'):
            zopepath = line.split('=')[1]
            zopepath = zopepath.strip('"\' ')
            return zopepath
    return ''

#------------------------------------------------------------------------------

def read_buildoutfile(buildoutfilename):
    """ read the buildout file to find the python path """
    try:
        zfile = open( buildoutfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % buildoutfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('#!'):
            zfile.close()
            return line.strip('#! ')
    return PYTHONBIN

#------------------------------------------------------------------------------

def testdb(fs, zopepath, fspath):
    """ call the tests scripts """
    pythonpath = os.path.join(zopepath, 'lib', 'python')
    fsfilename = os.path.join(fspath, fs)
    fstestfilename = os.path.join(zopepath, 'bin', 'fstest.py')
    if not os.path.exists(fstestfilename):
        fstestfilename = os.path.join(zopepath, 'utilities', 'ZODBTools', 'fstest.py')
    elif not os.path.exists(fstestfilename):
        fstestfilename = os.path.join(pythonpath, 'ZODB', 'scripts', 'fstest.py')
    fstestcmd = "env PYTHONPATH=%s %s %s "%(pythonpath, PYTHONBIN, fstestfilename)
    # -v : print a line by transaction
    # -vv : print a line by object
    start = datetime.now()
    verbose("\tTest of '%s'" % (fsfilename))
    cmd = fstestcmd + '%s' % (fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        verbose("\t\tError when testing : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tError when testing : '%s'" % "".join(cmd_out))
    verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

    if FSREFS:
        fsrefsfilename = os.path.join(zopepath, 'bin', 'fsrefs.py')
        if not os.path.exists(fsrefsfilename):
            fsrefsfilename = os.path.join(zopepath, 'utilities', 'ZODBTools', 'fsrefs.py')
        elif not os.path.exists(fsrefsfilename):
            fsrefsfilename = os.path.join(pythonpath, 'ZODB', 'scripts', 'fsrefs.py')
        fsrefscmd = "env PYTHONPATH=%s %s %s "%(pythonpath, PYTHONBIN, fsrefsfilename)
        # -v : print a line by transaction
        # -vv : print a line by object
        start = datetime.now()
        verbose("\tTest of '%s'" % (fsfilename))
        cmd = fsrefscmd + '%s' % (fsfilename)
        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)

        if cmd_err:
            verbose("\t\tError when testing refs : '%s'" % "".join(cmd_err))
        elif cmd_out:
            verbose("\t\tError when testing refs : '%s'" % "".join(cmd_out))
        verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

#------------------------------------------------------------------------------

def main():
    global buildout_inst_type, PYTHONBIN

    tmp = instdir
    if tmp.endswith('/'):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp,'parts')):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp,'etc')):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected"%tmp)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = instdir + '/parts/instance/etc/zope.conf'
        zopectlfilename = instdir + '/parts/instance/bin/zopectl'
        fspath = instdir + '/var/filestorage/'
        PYTHONBIN = read_buildoutfile(os.path.join(instdir, 'bin', 'buildout'))
    else:
        zodbfilename = instdir + '/etc/zope.conf'
        zopectlfilename = instdir + '/bin/zopectl'
        fspath = instdir + '/var/'

    # Getting some informations in config file
    (port, dbs) = treat_zopeconflines(zodbfilename)
    zopepath = read_zopectlfile(zopectlfilename)

    # Treating each db
    for db in dbs:
        testdb(db[1], zopepath, fspath)

#------------------------------------------------------------------------------

try:
    arg = sys.argv[1]
    if arg.startswith('#'):
        sys.exit(0)
    instdir, days, user, pwd = arg.split(';')
    verbose("Start of testing dbs of %s"%(instdir))
except IndexError:
    error("No parameter found")
    sys.exit(1)
except ValueError:
    error("No enough parameters")
    sys.exit(1)

if __name__ == '__main__':
    main()
    verbose("End of testing dbs of %s"%(instdir))



# -*- coding: utf-8 -*-
#
# File: Install.py
#
# Copyright (c) 2008 by CommunesPlone
# Generator: ArchGenXML Version 1.5.2
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan GEULETTE <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'


import os.path
import sys
from StringIO import StringIO
from sets import Set
from App.Common import package_home
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import manage_addTool
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from zExceptions import NotFound, BadRequest

from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.config import TOOL_NAME as ARCHETYPETOOLNAME
from Products.Archetypes.atapi import listTypes
from Products.CPUtils.config import PROJECTNAME
from Products.CPUtils.config import product_globals as GLOBALS

def install(self, reinstall=False):
    """ External Method to install CPUtils """
    out = StringIO()
    print >> out, "Installation log of %s:" % PROJECTNAME

    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    try:
        from Products.CPUtils.config import DEPENDENCIES
    except:
        DEPENDENCIES = []
    portal = getToolByName(self,'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)
        get_transaction().commit(1)

    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    install_subskin(self, out, GLOBALS)


    # try to call a workflow install method
    # in 'InstallWorkflows.py' method 'installWorkflows'
    try:
        installWorkflows = ExternalMethod('temp', 'temp',
                                          PROJECTNAME+'.InstallWorkflows',
                                          'installWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Install:'
        res = installWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow install'




    from Products.CPUtils.config import STYLESHEETS
    try:
        portal_css = getToolByName(portal, 'portal_css')
        for stylesheet in STYLESHEETS:
            try:
                portal_css.unregisterResource(stylesheet['id'])
            except:
                pass
            defaults = {'id': '',
            'media': 'all',
            'enabled': True}
            defaults.update(stylesheet)
            portal_css.registerStylesheet(**defaults)
    except:
        # No portal_css registry
        pass
    from Products.CPUtils.config import JAVASCRIPTS
    try:
        portal_javascripts = getToolByName(portal, 'portal_javascripts')
        for javascript in JAVASCRIPTS:
            try:
                portal_javascripts.unregisterResource(javascript['id'])
            except:
                pass
            defaults = {'id': ''}
            defaults.update(javascript)
            portal_javascripts.registerScript(**defaults)
    except:
        # No portal_javascripts registry
        pass

    # try to call a custom install method
    # in 'AppInstall.py' method 'install'
    try:
        install = ExternalMethod('temp', 'temp',
                                 PROJECTNAME+'.AppInstall', 'install')
    except NotFound:
        install = None

    if install:
        print >>out,'Custom Install:'
        try:
            res = install(self, reinstall)
        except TypeError:
            res = install(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom install'
    return out.getvalue()

def uninstall(self, reinstall=False):
    out = StringIO()



    # try to call a workflow uninstall method
    # in 'InstallWorkflows.py' method 'uninstallWorkflows'
    try:
        uninstallWorkflows = ExternalMethod('temp', 'temp',
                                            PROJECTNAME+'.InstallWorkflows',
                                            'uninstallWorkflows').__of__(self)
    except NotFound:
        uninstallWorkflows = None

    if uninstallWorkflows:
        print >>out, 'Workflow Uninstall:'
        res = uninstallWorkflows(self, out)
        print >>out, res or 'no output'
    else:
        print >>out,'no workflow uninstall'

    # try to call a custom uninstall method
    # in 'AppInstall.py' method 'uninstall'
    try:
        uninstall = ExternalMethod('temp', 'temp',
                                   PROJECTNAME+'.AppInstall', 'uninstall')
    except:
        uninstall = None

    if uninstall:
        print >>out,'Custom Uninstall:'
        try:
            res = uninstall(self, reinstall)
        except TypeError:
            res = uninstall(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom uninstall'

    return out.getvalue()

def beforeUninstall(self, reinstall, product, cascade):
    """ try to call a custom beforeUninstall method in 'AppInstall.py'
        method 'beforeUninstall'
    """
    out = StringIO()
    try:
        beforeuninstall = ExternalMethod('temp', 'temp',
                                   PROJECTNAME+'.AppInstall', 'beforeUninstall')
    except:
        beforeuninstall = []

    if beforeuninstall:
        print >>out, 'Custom beforeUninstall:'
        res = beforeuninstall(self, reinstall=reinstall
                                  , product=product
                                  , cascade=cascade)
        if res:
            print >>out, res
        else:
            print >>out, 'no output'
    else:
        print >>out, 'no custom beforeUninstall'
    return (out,cascade)

def afterInstall(self, reinstall, product):
    """ try to call a custom afterInstall method in 'AppInstall.py' method
        'afterInstall'
    """
    out = StringIO()
    try:
        afterinstall = ExternalMethod('temp', 'temp',
                                   PROJECTNAME+'.AppInstall', 'afterInstall')
    except:
        afterinstall = None

    if afterinstall:
        print >>out, 'Custom afterInstall:'
        res = afterinstall(self, product=None
                               , reinstall=None)
        if res:
            print >>out, res
        else:
            print >>out, 'no output'
    else:
        print >>out, 'no custom afterInstall'
    return out



#!/usr/bin/python
#
"""
    This script treats dbs of a zope instance :
    * packing each db
    * backuping with repozo each fs file
"""

import sys, urllib, os
import datetime
import shutil
from datetime import datetime 
from utils import *

buildout_inst_type = None #True for buildout, False for manual instance
zeo_type = False #True for zeo
BACKUP_DIR = '/srv/backups/zope'

#------------------------------------------------------------------------------

def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    ret = os.system(cmd + ' >_cmd_zr.out 2>_cmd_zr.err')
    stdout = stderr = []
    try:
        if os.path.exists('_cmd_zr.out'):
            ofile = open( '_cmd_zr.out', 'r')
            stdout = ofile.readlines()
            ofile.close()
            os.remove('_cmd_zr.out')
        else:
            error("File %s does not exist" % '_cmd_zr.out')
    except IOError:
        error("Cannot open %s file" % '_cmd_zr.out')
    try:
        if os.path.exists('_cmd_zr.err'):
            ifile = open( '_cmd_zr.err', 'r')
            stderr = ifile.readlines()
            ifile.close()
            os.remove('_cmd_zr.err')
        else:
            error("File %s does not exist" % '_cmd_zr.err')
    except IOError:
        error("Cannot open %s file" % '_cmd_zr.err')
    return( stdout,stderr )


    if not port:
        error("! the port was not found in the config file '%s'"%zodbfilename)

    return(port, dbs)

#------------------------------------------------------------------------------

def treat_zopeconflines(zodbfilename):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)
    zeo_fs = {}
    if zeo_type:
        zeoconffile = os.path.join(instdir, 'parts', 'zeo', 'etc', 'zeo.conf')
        lines2 = []
        read_zopeconffile(zeoconffile, lines2)
        fsflag = False
        fsnb = 0
        for line in lines2:
            if line.startswith('<filestorage '):
                fsflag = True
                fsnb = line.split()[1].strip(' >')
                #verbose("fsnb '%s'"%fsnb)
                continue
            if fsflag and line.startswith('path '):
                fsflag = False
                fs = line.split()[1]
                fs = os.path.basename(fs)
                #verbose("fs '%s'"%fs)
                if zeo_fs.has_key(fsnb):
                    error("Filestorage number '%s' already found"%fsnb)
                else:
                    zeo_fs[fsnb] = fs
                fsflag = False
                fsnb = 0
                continue
    port = ''
    httpflag = False
    fsflag = False
    dbs = []
    dbname = ''
    for line in lines:
        #verbose("=>'%s'"%line)
        if line.startswith('<http-server>'):
            httpflag = True
            continue
        if httpflag and line.startswith('address'):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith('<zodb_db'):
            if dbname:
                error("\tnext db found while fs not found: previous dbname '%s', current line '%s'"%(dbname, line))
            dbname = line.split()[1]
            dbname = dbname.rstrip('>')
            if dbname == 'temporary':
                dbname = ''
        if line.startswith('storage '): # ZEO
            fsnb = line.split()[1]
            if zeo_fs.has_key(fsnb):
                dbs.append([dbname, zeo_fs[fsnb]])
            else:
                error("Filestorage number '%s' not found in zeo.conf ?"%fsnb)
            dbname = ''
            continue
        if line.startswith('<filestorage>'): # NOT ZEO
            fsflag = True
            continue
        if fsflag and line.startswith('path'): # NOT ZEO
            fsflag = False
            fs = line.split()[1]
            fs = os.path.basename(fs)
            dbs.append([dbname, fs])
            dbname = ''
            continue
    verbose("\tport='%s', dbs='%s'"%(port, ';'.join([','.join(dbinfo) for dbinfo in dbs])))

    if not port:
        error("! the port was not found in the config file '%s'"%zodbfilename)

    return(port, dbs)

#------------------------------------------------------------------------------

def backupdb(fs, zopepath, fspath, pythonfile):
    """ call the repozo script to backup file """
    repozofilename = os.path.join(zopepath, 'bin', 'repozo.py')
    pythonpath = os.path.join(zopepath, 'lib', 'python')
    if not os.path.exists(repozofilename):
        repozofilename = os.path.join(instdir, 'bin', 'repozo')
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(zopepath, 'utilities', 'ZODBTools', 'repozo.py')
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(pythonpath, 'ZODB', 'scripts', 'repozo.py')
    backupcmd = "env PYTHONPATH=%s %s %s -Bv "%(pythonpath, pythonfile, repozofilename)
    if options.fullbackup:
        backupcmd += '-F '
    # -B / -R : backup or recover
    # -r backupdir
    # -F : full backup
    # -f fs file
    start = datetime.now()
    fsfilename = os.path.join(fspath, fs)
#        backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir))
#        if not os.path.exists(backupdir):
#            os.mkdir(backupdir)
    backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir), os.path.splitext(fs)[0])
    if os.path.exists(backupdir) and options.fullbackup:
        for file in shutil.os.listdir(backupdir):
            shutil.os.unlink(os.path.join(backupdir, file))
            verbose("\t%s deleted because we do a full backup" % (os.path.join(backupdir, file)))
    if not os.path.exists(backupdir):
        os.makedirs(backupdir)
    verbose("\tBackup of '%s' with script '%s'" % (fsfilename, repozofilename))    
    cmd = backupcmd + ' -r %s -f %s' % (backupdir, fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        verbose("\t\tOutput/Error when backuping : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tOutput when backuping : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)
    verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

#------------------------------------------------------------------------------

def main():
    global buildout_inst_type, zeo_type, instdir
    instance_section = 'instance'

    if instdir.endswith('/'):
        instdir = instdir[0:-1]
    verbose("Working on instance %s" % instdir)

    if os.path.exists(os.path.join(instdir,'parts')):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(instdir,'etc')):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(instdir) or True:
        error("! Invalid instance path '%s' or instance type not detected"%instdir)
        sys.exit(1)

    if buildout_inst_type:
        if os.path.exists(os.path.join(instdir, 'bin', 'zeo')):
            zeo_type = True
            verbose("\tInstance is a zeo !")
        if os.path.exists(os.path.join(instdir, 'bin', 'instance1')):
            instance_section = 'instance1'
        zodbfilename = instdir + '/parts/%s/etc/zope.conf'%instance_section
        zopectlfilename = instdir + '/parts/%s/bin/zopectl'%instance_section
        fspath = instdir + '/var/filestorage/'
    else:
        zodbfilename = instdir + '/etc/zope.conf'
        zopectlfilename = instdir + '/bin/zopectl'
        fspath = instdir + '/var/'

    # Getting some informations in config file
    (port, dbs) = treat_zopeconflines(zodbfilename)
    (zopepath, pythonfile) = read_zopectlfile(zopectlfilename)
    #verbose("repozo path='%s'"%zopepath)

    # We remove the folder containing all instance dbs in case of a full backup
    # It's necessary to avoid keeping old files to safe disk space
    if options.fullbackup:
        backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir))
        if os.path.exists(backupdir):
            shutil.rmtree(backupdir)
            verbose("\t%s deleted when full backup" % (backupdir))
    # Treating each db
    for db in dbs:
        packdb(port, db[0], days, 'cputils_pack_db', 'CPUtils.utils', 'pack_db', user,  pwd)
        #backupdb(db[1], zopepath, fspath, pythonfile)

    for file in shutil.os.listdir(fspath):
        if file.endswith('.fs.old'):
            shutil.os.unlink(fspath + file)
            verbose("\t%s deleted" % (fspath + file))
        elif file.endswith('.fs.pack'):
            shutil.os.unlink(fspath + file)
            error("! .pack file found : pack not correctly ended")
            verbose("\t%s deleted" % (fspath + file))

#------------------------------------------------------------------------------

try:
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-i", "--infos", dest="infos",
                  default=None,
                  help="infos about instance formatted like: "
                       "\"instance_path;transactions_days_number;admin_user;admin_password\"")
    parser.add_option("-F", "--full", dest="fullbackup",
                  help="do a full backup, not an incremental one",
                  default=False, action="store_true")
    (options, args) = parser.parse_args()
    if options.infos.startswith('#'):
        sys.exit(0)
    instdir, days, user, pwd = options.infos.split(';')
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == '__main__':
    verbose("Start of packing, backuping %s, days=%s, full_backup=%s"%(instdir, days, options.fullbackup))
#    sys.exit(0)
    main()
    verbose("End of packing, backuping %s"%(instdir))



#!/usr/bin/python
#
"""
    Restore of zope fs file backuped with the script pack_backup_db.py.
    This restore is done on a backup system where all instances files are copied in a RSYNCDIR dir
    Zope must be installed exactly at the same place on the main server and backup server
"""

import sys, urllib, os
import datetime
import shutil
from datetime import datetime 

buildout_inst_type = None #True for buildout, False for manual instance
BACKUP_DIR = '/srv/backups/zope'
RSYNCDIR = '' # prefix dir containing the copy of the rsync command
FSTEST = True
FSREFS = False
OVW_FS = False # overwrite fs file when restoring if already exist
PYTHONBIN = ''

def verbose(*messages):
    print '>>', ' '.join(messages)
def error(*messages):
#    print >>sys.stderr, '!!', (' '.join(messages))
    print '!!', (' '.join(messages))

#------------------------------------------------------------------------------

def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    ret = os.system(cmd + ' >_cmd_zr.out 2>_cmd_zr.err')
    stdout = stderr = []
    try:
        if os.path.exists('_cmd_zr.out'):
            ofile = open( '_cmd_zr.out', 'r')
            stdout = ofile.readlines()
            ofile.close()
            os.remove('_cmd_zr.out')
        else:
            error("File %s does not exist" % '_cmd_zr.out')
    except IOError:
        error("Cannot open %s file" % '_cmd_zr.out')
    try:
        if os.path.exists('_cmd_zr.err'):
            ifile = open( '_cmd_zr.err', 'r')
            stderr = ifile.readlines()
            ifile.close()
            os.remove('_cmd_zr.err')
        else:
            error("File %s does not exist" % '_cmd_zr.err')
    except IOError:
        error("Cannot open %s file" % '_cmd_zr.err')
    return( stdout,stderr )

#------------------------------------------------------------------------------

def read_zopeconffile(zodbfilename, lines):
    """ read the zope conf filename and include subfile """
    try:
        zfile = open( zodbfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % zodbfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('%include'):
            otherfilename = line.split()[1]
            if RSYNCDIR:
                if otherfilename.startswith('/'):
                    otherfilename = otherfilename[1:]
                otherfilename = os.path.join(RSYNCDIR, otherfilename)
            read_zopeconffile(otherfilename, lines)
            continue
        lines.append(line)

#------------------------------------------------------------------------------

def treat_zopeconflines(zodbfilename):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)

    port = ''
    httpflag = False
    fsflag = False
    dbs = []
    dbname = ''
    for line in lines:
        #verbose("=>'%s'"%line)
        if line.startswith('<http-server>'):
            httpflag = True
            continue
        if httpflag and line.startswith('address'):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith('<zodb_db'):
            if dbname:
                error("\tnext db found while fs not found: previous dbname '%s', current line '%s'"%(dbname, line))
            dbname = line.split()[1]
            dbname = dbname.rstrip('>')
            if dbname == 'temporary':
                dbname = ''
        if line.startswith('<filestorage>'):
            fsflag = True
            continue
        if fsflag and line.startswith('path'):
            fsflag = False
            fs = line.split()[1]
            fs = os.path.basename(fs)
            dbs.append([dbname, fs])
            dbname = ''
            continue
    verbose("\tport='%s', dbs='%s'"%(port, ';'.join([','.join(dbinfo) for dbinfo in dbs])))

    if not port:
        error("! the port was not found in the config file '%s'"%zodbfilename)

    return(port, dbs)

#------------------------------------------------------------------------------

def read_zopectlfile(zopectlfilename):
    """ read the zopectl file to find the zope path """
    try:
        zfile = open( zopectlfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % zopectlfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('ZOPE_HOME'):
            zopepath = line.split('=')[1]
            zopepath = zopepath.strip('"\' ')
            return zopepath
    return ''

#------------------------------------------------------------------------------

def read_buildoutfile(buildoutfilename):
    """ read the buildout file to find the python path """
    try:
        zfile = open( buildoutfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % buildoutfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('#!'):
            zfile.close()
            return line.strip('#! ')
    return PYTHONBIN

#------------------------------------------------------------------------------

def restoredb(fs, zopepath, fspath):
    """ call the repozo script to restore fs """
    repozofilename = os.path.join(zopepath, 'bin', 'repozo.py')
    pythonpath = os.path.join(zopepath, 'lib', 'python')
    if not os.path.exists(repozofilename):
        repozofilename = os.path.join(instdir, 'bin', 'repozo')
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(zopepath, 'utilities', 'ZODBTools', 'repozo.py')
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(pythonpath, 'ZODB', 'scripts', 'repozo.py')
    restorecmd = "env PYTHONPATH=%s %s %s -Rv "%(pythonpath, PYTHONBIN, repozofilename)
    # -B / -R : backup or recover
    # -r backupdir
    # -F : full backup
    # -f fs file to backup
    # -D date to restore
    # -o restored file name
    start = datetime.now()
    fsfilename = os.path.join(fspath, fs)
    backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir), os.path.splitext(fs)[0]) 
    if not OVW_FS and os.path.exists(fsfilename):
        error("\tfs file '%s' already exists ! Restoring in backup dir '%s' ..."%(fsfilename, backupdir))
        fsfilename = os.path.join(backupdir, fs)
        #return
    verbose("\tRestore of '%s'" % (fsfilename))    
    cmd = restorecmd + '-r %s -o %s' % (backupdir, fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        verbose("\t\tOutput/Error when restoring : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tOutput when restoring : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)
    verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

    if FSTEST:
        fstestfilename = os.path.join(zopepath, 'bin', 'fstest.py')
        if not os.path.exists(fstestfilename):
            fstestfilename = os.path.join(zopepath, 'utilities', 'ZODBTools', 'fstest.py')
        elif not os.path.exists(fstestfilename):
            fstestfilename = os.path.join(pythonpath, 'ZODB', 'scripts', 'fstest.py')
        fstestcmd = "env PYTHONPATH=%s %s %s "%(pythonpath, PYTHONBIN, fstestfilename)
        # -v : print a line by transaction
        # -vv : print a line by object
        start = datetime.now()
        verbose("\tTest of '%s'" % (fsfilename))
        cmd = fstestcmd + '%s' % (fsfilename)
        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)

        if cmd_err:
            error("\t\tError when testing : '%s'" % "".join(cmd_err))
        elif cmd_out:
            error("\t\tError when testing : '%s'" % "".join(cmd_out))
        else:
            verbose("\t\tAll seems OK")
        verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

    if FSREFS:
        fsrefsfilename = os.path.join(zopepath, 'bin', 'fsrefs.py')
        if not os.path.exists(fsrefsfilename):
            fsrefsfilename = os.path.join(zopepath, 'utilities', 'ZODBTools', 'fsrefs.py')
        elif not os.path.exists(fsrefsfilename):
            fsrefsfilename = os.path.join(pythonpath, 'ZODB', 'scripts', 'fsrefs.py')
        fsrefscmd = "env PYTHONPATH=%s %s %s "%(pythonpath, PYTHONBIN, fsrefsfilename)
        # -v : print a line by transaction
        # -vv : print a line by object
        start = datetime.now()
        verbose("\tTest of '%s'" % (fsfilename))
        cmd = fsrefscmd + '%s' % (fsfilename)
        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)

        if cmd_err:
            verbose("\t\tError when testing : '%s'" % "".join(cmd_err))
        elif cmd_out:
            verbose("\t\tError when testing : '%s'" % "".join(cmd_out))
        verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

#------------------------------------------------------------------------------

def copyfs(fs, fspath):
    """ copy the fs file in the right place """
    start = datetime.now()
    fsfilename = os.path.join(fspath, fs)
    backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir), os.path.splitext(fs)[0])
    backupfile = os.path.join(backupdir,fs)
#    if not os.path.exists(backupfile):
#        for filename in os.listdir(backupdir):
#            filepath = os.path.join(backupdir, filename)
#            if filepath.endswith('.fs'):
#                backupfile = filepath
#                break

#    if os.path.exists(fsfilename):
#        error("\tfs file '%s' already exists !"%(fsfilename))
#        return
    verbose("\tCopy of '%s' to '%s'" % (backupfile, fsfilename))    
    cmd = 'cp %s %s' % (backupfile, fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)
    if cmd_err:
        verbose("\t\tError when copying : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tError when copying : '%s'" % "".join(cmd_out))
    verbose("\t\t-> elapsed time %s"%(datetime.now()-start))

#------------------------------------------------------------------------------

def main():
    global buildout_inst_type, PYTHONBIN

    tmp = instdir
    if tmp.endswith('/'):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp,'parts')):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp,'etc')):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected"%tmp)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = instdir + '/parts/instance/etc/zope.conf'
        zopectlfilename = instdir + '/parts/instance/bin/zopectl'
        fspath = instdir + '/var/filestorage/'
        PYTHONBIN = read_buildoutfile(os.path.join(instdir, 'bin', 'buildout'))
    else:
        zodbfilename = instdir + '/etc/zope.conf'
        zopectlfilename = instdir + '/bin/zopectl'
        fspath = instdir + '/var/'

    # Getting some informations in config file
    (port, dbs) = treat_zopeconflines(zodbfilename)
    zopepath = read_zopectlfile(zopectlfilename)
    if RSYNCDIR:
        if zopepath.startswith('/'):
            zopepath = zopepath[1:]
        zopepath = os.path.join(RSYNCDIR, zopepath)
    verbose("zope path='%s'"%zopepath)

    # Treating each db
    for db in dbs:
        if RSYNCDIR:
            copyfs(db[1], fspath)
        else:
            restoredb(db[1], zopepath, fspath)

#------------------------------------------------------------------------------

try:
    arg = sys.argv[1]
    if arg.startswith('#'):
        sys.exit(0)
    instdir, days, user, pwd = arg.split(';')
    if RSYNCDIR:
        if instdir.startswith('/'):
            instdir = instdir[1:]
        instdir = os.path.join(RSYNCDIR, instdir)
    verbose("Start of restoring dbs of %s"%(instdir))
except IndexError:
    error("No parameter found")
    sys.exit(1)
except ValueError:
    error("No enough parameters")
    sys.exit(1)

if __name__ == '__main__':
    main()
    verbose("End of restoring dbs of %s"%(instdir))



from Products.CMFCore.utils import getToolByName

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('CPUtils')

from Products.CMFPlone.utils import base_hasattr
from Products.CPUtils.config import *

def setRobots(context):
    """
        Copy a robots.txt file from files dir to portal_skins/custom
    """
    if context.readDataFile('cputils_robots.txt') is None:
        return

    portal = context.getSite()
    output = []

    # Add additional setup code here

    zopeFolder = portal.portal_skins.custom

    filename = name = 'robots.txt'
    path = os.path.join(FILES_FOLDER, filename)
    fd = open(path, 'r')
    data = fd.read()
    fd.close()

    # if the file exists and title starts with Remove ..., we replace the file
    if base_hasattr(zopeFolder, name):
        rf = getattr(zopeFolder, name)
        if rf.title.startswith('Remove this file in production mode!'):
            zopeFolder.manage_delObjects([name])

    if not base_hasattr(zopeFolder, name):
        output.append("Creating file '%s' in '%s'"%(name, 'portal_skins/custom'))
        zopeFolder.manage_addProduct['OFSP'].manage_addFile(id=name, file=data, title='Remove this file in production mode! A effacer une fois le site en production!')
        if name in zopeFolder.objectIds():
            output.append("File created")
        else:
            output.append("File NOT created")
    else:
        output.append("File '%s' in '%s' already exists"%(name, 'portal_skins/custom'))

    return '\n'.join(output)



# -*- coding: utf-8 -*-
#
# File: CPUtils.py
#
# Copyright (c) 2008 by CommunesPlone
# Generator: ArchGenXML Version 1.5.2
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan GEULETTE <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'


# Product configuration.
#
# The contents of this module will be imported into __init__.py, the
# workflow configuration and every content type module.
#
# If you wish to perform custom configuration, you may put a file
# AppConfig.py in your product's root directory. This will be included
# in this file if found.

try:  # New CMF
    from Products.CMFCore.permissions import setDefaultRoles
except ImportError:  # Old CMF
    from Products.CMFCore.CMFCorePermissions import setDefaultRoles


##code-section config-head #fill in your manual code here
##/code-section config-head


PROJECTNAME = "CPUtils"

# Check for Plone 2.1
try:
    from Products.CMFPlone.migrations import v2_1
except ImportError:
    HAS_PLONE21 = False
else:
    HAS_PLONE21 = True

# Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner'))

product_globals = globals()

# Dependencies of Products to be installed by quick-installer
# override in custom configuration
DEPENDENCIES = []

# Dependend products - not quick-installed - used in testcase
# override in custom configuration
PRODUCT_DEPENDENCIES = []

# You can overwrite these two in an AppConfig.py:
# STYLESHEETS = [{'id': 'my_global_stylesheet.css'},
#                {'id': 'my_contenttype.css',
#                 'expression': 'python:object.getTypeInfo().getId() == "MyType"'}]
# You can do the same with JAVASCRIPTS.
STYLESHEETS = []
JAVASCRIPTS = []

##code-section config-bottom #fill in your manual code here
import os
PRODUCT_FOLDER = os.path.dirname(__file__)
FILES_FOLDER = os.path.join(PRODUCT_FOLDER, 'data')


def getPloneVersion():
    import Products.CMFPlone as cmfp
    plonedir = cmfp.__path__[0]
    # 2.5, 3 version
    if os.path.exists(plonedir):
        for name in ('version.txt', 'VERSION.txt', 'VERSION.TXT'):
            versionfile = os.path.join(plonedir, name)
            if os.path.exists(versionfile):
                file = open(versionfile, 'r')
                data = file.readline()
                file.close()
                return data.strip()
    # 4 version
    import pkg_resources
    return pkg_resources.get_distribution('Products.CMFPlone').version

PLONE_VERSION = getPloneVersion()

##/code-section config-bottom


# Load custom configuration not managed by ArchGenXML
try:
    from Products.CPUtils.AppConfig import *
except ImportError:
    pass



# -*- coding: utf-8 -*-
#
# File: CPUtils.py
#
# Copyright (c) 2008 by CommunesPlone
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan GEULETTE <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

import logging
logger = logging.getLogger('CPUtils')
logger.info('Installing Product')

from Acquisition import aq_base
from ZPublisher.HTTPRequest import FileUpload
from Products.CMFQuickInstallerTool.QuickInstallerTool import QuickInstallerTool
from Products.CMFCore.utils import getToolByName
try:
    from Products.validation.i18n import PloneMessageFactory as _
    from Products.validation.i18n import recursiveTranslate
    from Products.validation.i18n import safe_unicode
except:
    pass
from Products.validation.validators.SupplValidators import MaxSizeValidator

try:
    from Products.CMFQuickInstallerTool.utils import get_packages
except:
    pass

from Products.CMFCore import DirectoryView
#from Products.Archetypes.atapi import *
#from Products.Archetypes import listTypes
from config import product_globals, PLONE_VERSION

DirectoryView.registerDirectory('skins', product_globals)
DirectoryView.registerDirectory('skins/CPUtils', product_globals)


def getQIFilteringInformation(self):
    from AccessControl.SecurityManagement import getSecurityManager
    doFiltering = True
    hiddenProducts = ['def']
    shownProducts = ['def']
    user = getSecurityManager().getUser()
    portal = getToolByName(self, 'portal_url').getPortalObject()
    if user.__module__ == 'Products.PluggableAuthService.PropertiedUser':
        doFiltering = False
    if hasattr(portal, 'hiddenProducts'):
        hp = list(portal.hiddenProducts)
        hiddenProducts += [p.strip() for p in hp]
    if hasattr(portal, 'shownProducts'):
        sp = list(portal.shownProducts)
        shownProducts += [p.strip() for p in sp]
    return doFiltering, hiddenProducts, shownProducts


def listInstallableProducts25(self, skipInstalled=1):
    """List candidate CMF products for
    installation -> list of dicts with keys: (id, hasError, status)
    """
    try:
        from zpi.zope import not_installed
        #print 'Packman support(hotplug) installed'
    except ImportError:
        def not_installed(s):
            return []

    # reset the list of broken products
    self.errors = {}
    pids = self.Control_Panel.Products.objectIds() + not_installed(self)
    pids = [pid for pid in pids if self.isProductInstallable(pid)]

    if skipInstalled:
        installed = [p['id'] for p in self.listInstalledProducts(showHidden=1)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        p = self._getOb(r, None)
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        if p:
            res.append({'id': r, 'status': p.getStatus(),
                        'hasError': p.hasError()})
        else:
            res.append({'id': r, 'status': 'new', 'hasError': 0})
    res.sort(lambda x, y: cmp(x.get('id', None), y.get('id', None)))
    return res


def listInstalledProducts25(self, showHidden=0):
    """Returns a list of products that are installed -> list of
    dicts with keys: (id, hasError, status, isLocked, isHidden,
    installedVersion)
    """
    pids = [o.id for o in self.objectValues()
            if o.isInstalled() and (o.isVisible() or showHidden)]

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        p = self._getOb(r, None)
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        res.append({'id': r, 'status': p.getStatus(),
                    'hasError': p.hasError(),
                    'isLocked': p.isLocked(),
                    'isHidden': p.isHidden(),
                    'installedVersion': p.getInstalledVersion()})
    res.sort(lambda x, y: cmp(x.get('id', None), y.get('id', None)))
    return res


def listInstallableProducts31(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys: (id, title, hasError, status)
    """
    # reset the list of broken products
    self.errors = {}

    # Get product list from control panel
    pids = self.Control_Panel.Products.objectIds()
    pids = [p for p in pids if self.isProductInstallable(p)]

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()
    profile_pids = [p for p in profile_pids if self.isProductInstallable(p)]
    for p in profile_pids:
        if p.startswith('Products.'):
            p = p[9:]
        if p not in pids:
            pids.append(p)

    if skipInstalled:
        installed = [p['id'] for p in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile['title']
        if p:
            res.append({'id': r, 'title': name, 'status': p.getStatus(),
                        'hasError': p.hasError()})
        else:
            res.append({'id': r, 'title': name, 'status': 'new', 'hasError': False})
    res.sort(lambda x, y: cmp(x.get('title', x.get('id', None)),
                              y.get('title', y.get('id', None))))
    return res


def listInstalledProducts31(self, showHidden=False):
    """Returns a list of products that are installed -> list of
    dicts with keys: (id, title, hasError, status, isLocked, isHidden,
    installedVersion)
    """
    pids = [o.id for o in self.objectValues()
            if o.isInstalled() and (o.isVisible() or showHidden)]
    pids = [pid for pid in pids if self.isProductInstallable(pid)]

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile['title']

        res.append({'id': r,
                    'title': name,
                    'status': p.getStatus(),
                    'hasError': p.hasError(),
                    'isLocked': p.isLocked(),
                    'isHidden': p.isHidden(),
                    'installedVersion': p.getInstalledVersion()})
    res.sort(lambda x, y: cmp(x.get('title', x.get('id', None)),
                              y.get('title', y.get('id', None))))
    return res


def listInstallableProducts40(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys: (id, title, hasError, status)
    """
    # reset the list of broken products
    try:
        self.errors = {}
    except AttributeError:
        pass

    # Returns full names with Products. prefix for all packages / products
    packages = get_packages()

    pids = []
    for p in packages:
        if not self.isProductInstallable(p):
            continue
        if p.startswith('Products.'):
            p = p[9:]
        pids.append(p)

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()

    for p in profile_pids:
        if p in pids or p in packages:
            continue
        if not self.isProductInstallable(p):
            continue
        pids.append(p)

    if skipInstalled:
        installed = [p['id'] for p in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile['title']
        if p:
            res.append({'id': r, 'title': name, 'status': p.getStatus(),
                        'hasError': p.hasError()})
        else:
            res.append({'id': r, 'title': name, 'status': 'new', 'hasError': False})
    res.sort(lambda x, y: cmp(x.get('title', x.get('id', None)),
                              y.get('title', y.get('id', None))))
    return res


def listInstallableProducts434(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys: (id, title, hasError, status)
    """
    # reset the list of broken products
    if getattr(self, '_v_errors', True):
        self._v_errors = {}

    # Returns full names with Products. prefix for all packages / products
    packages = get_packages()

    pids = []
    for p in packages:
        if not self.isProductInstallable(p):
            continue
        if p.startswith('Products.'):
            p = p[9:]
        pids.append(p)

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()

    for p in profile_pids:
        if p in pids or p in packages:
            continue
        if not self.isProductInstallable(p):
            continue
        pids.append(p)

    if skipInstalled:
        installed = [p['id'] for p in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile['title']
        if p:
            res.append({'id': r, 'title': name, 'status': p.getStatus(),
                        'hasError': p.hasError()})
        else:
            res.append({'id': r, 'title': name, 'status': 'new', 'hasError': False})
    res.sort(lambda x, y: cmp(x.get('title', x.get('id', None)),
                              y.get('title', y.get('id', None))))
    return res


def listInstallableProducts437(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys:(id,title,hasError,status)
    """
    self._init_errors(reset=True)

    # Returns full names with Products. prefix for all packages / products
    packages = get_packages()

    pids = []
    for pkg in packages:
        if not self.isProductInstallable(pkg):
            continue
        if pkg.startswith('Products.'):
            pkg = pkg[9:]
        pids.append(pkg)

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()

    for pp in profile_pids:
        if pp in pids or pp in packages:
            continue
        if not self.isProductInstallable(pp):
            continue
        pids.append(pp)

    from Products.CPUtils.__init__ import getQIFilteringInformation
    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    if skipInstalled:
        installed = [
            p['id'] for p in self.listInstalledProducts(showHidden=True)
        ]
        pids = [r for r in pids if r not in installed]

    res = []
    for pid in pids:
        if doFiltering and pid in hiddenProducts and pid not in shownProducts:
            continue
        installed_product = self._getOb(pid, None)
        name = pid
        profile = self.getInstallProfile(pid)
        if profile:
            name = profile['title']
        record = {'id': pid, 'title': name}
        if installed_product:
            record['status'] = installed_product.getStatus()
            record['hasError'] = installed_product.hasError()
        else:
            record['status'] = 'new'
            record['hasError'] = False
        res.append(record)
    res.sort(
        lambda x, y: cmp(
            x.get('title', x.get('id', None)),
            y.get('title', y.get('id', None))
        )
    )
    return res


def CallMaxSizeValidator(self, value, *args, **kwargs):
        instance = kwargs.get('instance', None)
        field = kwargs.get('field', None)
        type_doc = instance.getPortalTypeName().replace(' ', '').lower() + '_maxsize'
        # get max size
        if 'maxsize' in kwargs:
            maxsize = kwargs.get('maxsize')
        elif hasattr(aq_base(instance), 'getMaxSizeFor'):
            maxsize = instance.getMaxSizeFor(field.getName())
        elif hasattr(field, 'maxsize'):
            maxsize = field.maxsize
        elif hasattr(instance, type_doc):
            maxsize = getattr(instance, type_doc)
        else:
            # set to given default value (default defaults to 0)
            maxsize = self.maxsize

        if not maxsize:
            return True

        # calculate size
        elif (isinstance(value, FileUpload) or isinstance(value, file) or
              hasattr(aq_base(value), 'tell')):
            value.seek(0, 2)  # eof
            size = value.tell()
            value.seek(0)
        else:
            try:
                size = len(value)
            except TypeError:
                size = 0
        size = float(size)
        sizeMB = (size / (1024 * 1024))

        if sizeMB > maxsize:
            msg = _("Validation failed($name: Uploaded data is too large: ${size}MB (max ${max}MB)",
                    mapping={
                        'name': safe_unicode(self.name),
                        'size': safe_unicode("%.3f" % sizeMB),
                        'max': safe_unicode("%.3f" % maxsize)
                    })
            return recursiveTranslate(msg, **kwargs)
        else:
            return True


def initialize(context):
    logger.info("ADDING MONKEY PATCHS !")
    if not PLONE_VERSION:
        logger.error('CMFPlone version NOT FOUND: MONKEY PATCH NOT APPLIED')
        return
    elif PLONE_VERSION.startswith('2.5'):
        QuickInstallerTool.listInstallableProducts = listInstallableProducts25
        QuickInstallerTool.listInstalledProducts = listInstalledProducts25
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
    elif PLONE_VERSION.startswith('3.'):
        QuickInstallerTool.listInstallableProducts = listInstallableProducts31
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
    elif PLONE_VERSION < '4.3.4':
        QuickInstallerTool.listInstallableProducts = listInstallableProducts40
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        try:
            #Patching tinymce to load as html in Ploneboard
            from Products.TinyMCE.utility import TinyMCE
            old_getContentType = TinyMCE.getContentType

            def getContentType(self, object=None, field=None, fieldname=None):
                if object is not None and fieldname == 'text' and object.meta_type in ('PloneboardForum',
                                                                                       'PloneboardConversation'):
                    return 'text/html'
                return old_getContentType(self, object=object, field=field, fieldname=fieldname)
            #TinyMCE.getContentType = getContentType
            logger.info("TinyMCE getContentType MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        except:
            pass
    elif PLONE_VERSION >= '4.3.4':
        QuickInstallerTool.listInstallableProducts = listInstallableProducts434
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
    elif PLONE_VERSION >= '4.3.7':
        QuickInstallerTool.listInstallableProducts = listInstallableProducts437
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)



# -*- coding: utf-8 -*-
#
# File: testMethods.py
#
# Copyright (c) 2007 by CommunesPlone
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan Geulette <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

import os, sys
from zExceptions import NotFound
from Products.ExternalMethod.ExternalMethod import ExternalMethod
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))
import transaction
from AccessControl import getSecurityManager, ClassSecurityInfo
from AccessControl.SecurityManagement import newSecurityManager, setSecurityManager

#
# Test-cases for class(es) 
#

from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
from Products.CPUtils.config import *
from Products.CPUtils.tests.CPUtilsTestCase import CPUtilsTestCase

# Import the tested classes

##code-section module-beforeclass #fill in your manual code here
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
##/code-section module-beforeclass
ZopeTestCase.installProduct(PROJECTNAME)

PRODUCTS = list()
PRODUCTS.append(PROJECTNAME)
productName = "PloneGazette"
PRODUCTS.append(productName)

testcase = PloneTestCase.PloneTestCase

##code-section module-before-plone-site-setup #fill in your manual code here

##/code-section module-before-plone-site-setup

#Extension profile doesn't work because appinstall of ts appears to be not run
#PloneTestCase.setupPloneSite(products=PRODUCTS, extension_profiles=EXTENSION_PROFILES)
PloneTestCase.setupPloneSite(products=PRODUCTS)


class testNewsLetter(CPUtilsTestCase):
    """Test-cases for class(es) ."""
    
    ##code-section class-header_testMethods #fill in your manual code here
    ##/code-section class-header_testMethods

    def afterSetUp(self):
        from Products.PloneGazette import NewsletterTheme
        """
        """        
        CPUtilsTestCase.afterSetup(self)
        
        self.loginAsPortalOwner()
        method = 'install'
        method_name = 'cputils_'+method
        manage_addExternalMethod(self.app, method_name, '', 'CPUtils.utils', method)
        self.app.cputils_install()
        
        """from Products.Five import zcml
        from Products.Five import fiveconfigure
        fiveconfigure.debug_mode=True
        import Products.PloneGazette
        zcml.load_config('configure.zcml', Products.PloneGazette)
        fiveconfigure.debug_mode=False
        self.addProduct(name)"""
        qi = self.portal.portal_quickinstaller
        #print(productName+" installed : ")
        #print(qi.isProductInstalled(productName))
        
    # Manually created methods
    
    def test_list_newsletter_users(self):  
        from Products.PloneGazette.NewsletterTheme import NewsletterTheme
        
        print("problème pour créer une newsletter et ses abonnés: invokeFactory() ne fonctionne pas ")
        print("(probablement car plonegazette ne s'installe pas correctement)")
        """
        self.portal.invokeFactory('NewsletterTheme', id)
        nlf = self.portal.newsletter

        nlf.invokeFactory('NewsletterBTree', id='subscribers')
        nlf.subscriber_folder_id = 'subscribers'

        nlf.createSubscriberObject('sub1')
        nlf.createSubscriberObject('sub2')
        nlf.createSubscriberObject('sub3')
        subList = nlf.getSubscribers()
        print(subList)"""
        self.failUnless(True)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testNewsLetter))
    return suite

##code-section module-footer #fill in your manual code here
##/code-section module-footer

if __name__ == '__main__':
    framework()



#!/usr/bin/python
#

import sys, urllib, os
import datetime
import shutil
from utils import *

buildout_inst_type = None #True for buildout, False for manual instance

#------------------------------------------------------------------------------

def main():
    global buildout_inst_type

    tmp = instdir
    if tmp.endswith('/'):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp,'parts')):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp,'etc')):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected"%tmp)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = instdir + '/parts/instance/etc/zope.conf'
    else:
        zodbfilename = instdir + '/etc/zope.conf'

    lines = []
    read_zopeconffile(zodbfilename, lines)

    port = ''
    httpflag = False
    dbs = []
    for line in lines:
        #verbose("=>'%s'"%line)
        if line.startswith('<http-server>'):
            httpflag = True
            continue
        if httpflag and line.startswith('address'):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith('<zodb_db'):
            dbname = line.split()[1]
            dbname = dbname.rstrip('>')
            if dbname != 'temporary':
                dbs.append(dbname)
    verbose("\tport='%s', dbs='%s'"%(port, ';'.join(dbs)))

    if not port:
        error("! the port was not found in the config file '%s'"%zodbfilename)

    packdb(port, dbs, days, 'cputils_pack_db', 'CPUtils.utils', 'pack_db')
    
    #we delete every .old created during packing in the /var directory of the zope instance
    if buildout_inst_type:
        dir_path = instdir + '/var/filestorage/'
    else:
        dir_path = instdir + '/var/'

    for file in shutil.os.listdir(dir_path):
        if file.endswith('.fs.old'):
            #we do no more delete fs.old to safely backup this file as zodb file
            #shutil.os.unlink(dir_path + file)
            #verbose("\t%s deleted" % (dir_path + file))
            pass
        elif file.endswith('.fs.pack'):
            shutil.os.unlink(dir_path + file)
            error("! .pack file found : pack not correctly ended")
            verbose("\t%s deleted" % (dir_path + file))

#------------------------------------------------------------------------------

try:
    arg = sys.argv[1]
    if arg.startswith('#'):
        sys.exit(0)
    instdir, days, user, pwd = arg.split(';')
    verbose("Start of packing %s, days=%s"%(instdir, days))
except IndexError:
    error("No parameter found")
    sys.exit(1)
except ValueError:
    error("No enough parameters")
    sys.exit(1)

if __name__ == '__main__':
    main()
    verbose("End of packing %s"%(instdir))



# -*- coding: utf-8 -*-
#
# File: config.py
#
# Copyright (c) 2006 by CommunesPlone
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan GEULETTE <stephan.geulette@uvcw.be>, Gauthier BASTIEN <gbastien@commune.sambreville.be>"""
__docformat__ = 'plaintext'

from Products.CMFCore.utils import getToolByName

from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
import logging

logger = logging.getLogger('CPUtils')

def install(self, reinstall=False):
    """ 
        Specific installation steps
    """
    portal = getToolByName(self, 'portal_url').getPortalObject()
    #Add of some external methods
    # pack_db method must not be added here
    for method in ('object_info', 'audit_catalog'):
        method_name = 'cputils_'+method
        if not hasattr(portal.aq_inner.aq_explicit, method_name):
            logger.info("REALLY Adding external method '%s'" % method_name)
            #without aq_explicit, if the id exists at a higher level, it is found !
            manage_addExternalMethod(self, method_name, '', 'CPUtils.utils', method)
    
def uninstall(self, reinstall=False):
    """
        Specific uninstallation steps
    """
    portal = getToolByName(self, 'portal_url').getPortalObject()
    #remove the external methods
    for method in ('object_info', 'audit_catalog'):
        method_name = 'cputils_'+method
        try:
            portal.manage_delObjects(method_name)
            logger.info("Deleted external method '%s'" % method_name)
        except AttributeError:
            logger.warn("The '%s' ExternalMethod could not be found, we continue..." % method_name)




global dic_hpList
dic_hpList = {
'acptheme.cpskin3':[
  'ATVocabularyManager',
  'CacheSetup',
  'CMFSquidTool',
  'Clouseau',
  'CPUtils',
  'ImageEditor',
  'LinguaPlone',
  'Marshall',
  'NuPlone',
  'Ploneboard',
  'RICDescriptive',
  'SimpleAttachment',
  'SmartColorWidget',
  'ZopeRepository',
  'collective.js.jquery',
  'collective.js.jqueryui',
  'plone.app.iterate',
  'plone.app.openid',
  'plone.app.z3cform',
],
'urban':[
 'produit 1',
 'produit 3',
 'produit 4'
],
'PloneMeeting:':[
    
],
'TeleServices':[
    
]
}




# -*- coding: utf-8 -*-
#
# File: testMethods.py
#
# Copyright (c) 2007 by CommunesPlone
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan Geulette <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

import os, sys
from zExceptions import NotFound
from Products.ExternalMethod.ExternalMethod import ExternalMethod
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))
import transaction
##code-section module-header #fill in your manual code here
##/code-section module-header

#
# Test-cases for class(es) 
#

from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
from Products.CPUtils.config import *
from Products.CPUtils.tests.CPUtilsTestCase import CPUtilsTestCase

# Import the tested classes

##code-section module-beforeclass #fill in your manual code here
from sets import Set
from AccessControl.SecurityManagement import getSecurityManager
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
##/code-section module-beforeclass


class testMethods(CPUtilsTestCase):
    """Test-cases for class(es) ."""

    ##code-section class-header_testMethods #fill in your manual code here
    ##/code-section class-header_testMethods

    def afterSetUp(self):
        """
        """        
        CPUtilsTestCase.afterSetup(self)
        
        self.loginAsPortalOwner()
        method = 'install'
        method_name = 'cputils_'+method
        manage_addExternalMethod(self.app, method_name, '', 'CPUtils.utils', method)
        self.app.cputils_install()
        
    # Manually created methods
    
        
    def test_audit_catalog(self):
        result = self.portal.cputils_audit_catalog()
        ok = result.startswith("<h1>RESULTATS DE LA RECHERCHE</h1> <p>Recherche dans : /</p> <p>Nombre d'elements trouves :")
        ok = ok and result.find("portal_type : getObjSize : url") > -1
        self.failUnless(ok)        
        
    def test_change_authentication_plugins_param_doChange (self):
        result = self.app.cputils_change_authentication_plugins(self.app,'','')
        ok = result.startswith("The following changes are not applied: you must run the script with the parameter '...?dochange=1'")
        result = self.app.cputils_change_authentication_plugins(self.app,1,'')
        ok = ok and result.startswith("The following changes are not applied: you must run the script with the parameter '...?dochange=1'")
        self.failUnless(ok)
        
    def test_change_authentication_plugins_param_activate (self):
        result = self.app.cputils_change_authentication_plugins(self.app,'',1)
        ok = result.startswith("Document '/authentication_plugins_sites' added")
        ok = ok and result.find("Desactivate plugins source_users") > -1 
        result = self.app.cputils_change_authentication_plugins(self.app,1,1)
        ok = ok and result.find("site found plone")
        ok = ok and result.find("Activate plugins source_users") > -1
        self.failUnless(ok)
        
    def test_change_user_properties_no_param(self):
        result = self.portal.cputils_change_user_properties(self.portal,'','')
        ok = result.find("USER:'member'") > -1 and result.find("USER:'admin'") > -1 and result.find("USER:'anon'") > -1
        self.failUnless(ok)
        
    def test_change_user_properties_param_visibleIDs(self):
        result = self.portal.cputils_change_user_properties(self.portal,'visible_ids:True','')
        ok = result.find("USER:'member'") > -1 and result.find("USER:'admin'") > -1 and result.find("USER:'anon'") > -1
        ok = ok and result.find(" all properties: visible_ids='False'")
        result = self.portal.cputils_change_user_properties(self.portal,'visible_ids:True',1)
        ok = result.find("USER:'member'") > -1 and result.find("USER:'admin'") > -1 and result.find("USER:'anon'") > -1
        ok = ok and result.find("old properties: visible_ids='False',<br/>->  new properties: visible_ids='True',") 
        self.failUnless(ok)
        
    """def test_checkInstance(self):
        result = self.portal.cputils_checkInstance()
        self.failUnless(ok)
    
    def test_checkPOSKey(self):
        self.failUnless(False)
        
    def test_configure_fckeditor(self):
            self.failUnless(False)
        
    def test_copy_image_attribute(self):
        self.failUnless(False)"""
        
    def test_cpdb(self):
        print("push 'c' and then 'return' to pass the test")
        self.portal.cputils_cpdb()
        "cputils_cpdb test passed"
        
    def test_desactivate_base2dom(self):
        result = self.app.cputils_desactivate_base2dom()
        ok = result.find("Disabled ++resource++base2-dom-fp.js for /plone") > -1
        self.failUnless(ok)
        
    def test_install_plone_product(self):
        #TEST_INSTALL_PLONE_PRODUCT
        #DEPRECATION WARNING: QUICKINSTALLER WILL BE UNSUPORTED IN THE NEXT VERSIONS
        self.failUnless(True)
        
    def test_list_portlets(self):
        result = self.portal.cputils_list_portlets()
        ok = result.startswith("left: {u'login': <Assignment at login>, u'navigation': <Assignment at navigation>}")
        ok = ok and result.endswith("right: {u'news': <Assignment at news>, u'review': <Assignment at review>, u'events': <Assignment at events>, u'calendar': <Assignment at calendar>}")
        self.failUnless(ok)
        
    def test_list_users_param_sort(self):
        result = self.portal.cputils_list_users(self.portal,'csv','users')
        ok = result.endswith("<br />UserId,GroupId<br />admin,AuthenticatedUsers<br />member,AuthenticatedUsers<br />anon,AuthenticatedUsers<br />test_user_1_,AuthenticatedUsers")
        #result = self.portal.cputils_list_users('csv','groups')
        self.failUnless(ok)
        
    """def test_load_user_properties(self):
        self.failUnless(False)
        
    def test_object_info(self):
        self.failUnless(False)
        
    def test_recreate_users_groups(self):
        self.failUnless(False)
        
    def test_rename_long_ids(self):
        self.failUnless(False)
        
    def test_send_adminMail(self):
        self.failUnless(False)"""
            
    def test_store_user_properties(self): 
        result = self.portal.cputils_store_user_properties()
        ok = str(self.portal.users_properties).find("False	Kupu") > -1
        self.portal.cputils_change_user_properties(self.portal,'visible_ids:True',1)
        result = self.portal.cputils_store_user_properties()
        ok = ok and str(self.portal.users_properties).find("True	Kupu") > -1
        self.failUnless(ok)
        
    """def test_sync_properties(self):
        self.failUnless(False)  """   


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testMethods))
    return suite

##code-section module-footer #fill in your manual code here
##/code-section module-footer

if __name__ == '__main__':
    framework()





#!/usr/bin/python
#
"""
    This script reads the log file of all scripts and send a mail with the day content.
"""

import os
import datetime
import sys

def verbose(*messages):
    print '>>', ' '.join(messages)
def error(*messages):
#    print >>sys.stderr, '!!', (' '.join(messages))
    print '!!', (' '.join(messages))

#------------------------------------------------------------------------------

MAIL = "/usr/sbin/sendmail"
logfilenames = ('/var/log/cron_scripts.log', '/var/log/checkInstances.log', '/var/log/checkPoskey.log')
#logfilenames = ('logs/cron_scripts.log','logs/checkInstances.log')
hostname = 'mydomain.be'
today = datetime.date.today().strftime("%Y-%m-%d")
yesterday = (datetime.date.today()-datetime.timedelta(1)).strftime("%Y-%m-%d")
header = (  'To: server@communesplone.be', 
            'From: root@%s'%hostname,)

#------------------------------------------------------------------------------

def read_log_file(filename, lines):
    """
        Read the log file
    """
    try:
        logfile = open( filename, 'r')
        verbose("Reading file '%s'"%filename)
    except IOError:
        error("Cannot open %s file" % filename)
        return
    flag_tokeep = False
    flag_yesterday = False
    for line in logfile.readlines():
        line = line.strip('\n\t ')
        #print line
        if line.startswith("##  %s, 23"%yesterday):
            flag_yesterday = True
            flag_tokeep = True
        if line.startswith("##  %s,"%today):
#            if not flag_yesterday:
#                read_log_file(filename+'.1', lines)
            flag_tokeep = True
        if flag_tokeep:
            lines.append(line)
#    if not flag_tokeep:
#        read_log_file(filename+'.1', lines)

#------------------------------------------------------------------------------

verbose("Searching in log between %s and %s"%(yesterday, today))

for logfilename in logfilenames:
    bodylines = ['']
    read_log_file(logfilename, bodylines)
    # when logrotate is run, the logfile is cleared and 
    # renamed in .1 (with 'delaycompress' option for logrotate)
    # we open the .1 file to get those lines
    if len(bodylines) < 10 and os.path.exists(logfilename+'.1'):
        rotatedlines = ['']
        read_log_file(logfilename+'.1', rotatedlines)
        bodylines[0:0] = rotatedlines

    #print '\n'.join(bodylines)
    #sys.exit(0)

    subject = '\nSubject: %s %s from %s'%(today, os.path.basename(logfilename), hostname)

    # open a pipe to the mail program and
    # write the data to the pipe
    p = os.popen("%s -t" % MAIL, 'w')
#    p = open("%s"%os.path.basename(logfilename), 'w')  #testing
    p.write('\n'.join(header))
    p.write(subject)
    p.write('\n'.join(bodylines))
    exitcode = p.close()
    if exitcode:
        print "Exit code: %s" % exitcode



# -*- coding: utf-8 -*-
#
# File: test_functional.py
#
# Copyright (c) 2007 by CommunesPlone
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# functional doctests.  This module collects all *.txt
# files in the tests directory and runs them.

__author__ = """Stephan Geulette <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import glob
from zope.testing import doctest
import unittest
from Globals import package_home
from Testing.ZopeTestCase import FunctionalDocTestSuite
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite
from Products.CMFPlone.tests import PloneTestCase
from Testing import ZopeTestCase
from Products.CPUtils.tests.CPUtilsTestCase import CPUtilsTestCase

PACKAGE = 'Products.CPUtils.tests'

REQUIRE_TESTBROWSER = ['tst_CPUtils.txt']

OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

def list_doctests():
    home = package_home(globals())
    print home
    return [filename for filename in
            glob.glob(os.path.sep.join([home, '*.txt']))]

def list_nontestbrowser_tests():
    return [filename for filename in list_doctests()
            if os.path.basename(filename) not in REQUIRE_TESTBROWSER]

def setUp(self):
    uf = self.portal.acl_users
    uf.userFolderAddUser('member', 'member', ['Member', ], [])
    uf.userFolderAddUser('admin', 'admin', ['Manager', 'Member', ], [])
    uf.userFolderAddUser('anon', 'anon', ['Anonymous', ], [])
    #we have to be logged as a Manager to run methods from exportimport
    self.login('admin')
    self.logout()
    return None

def test_suite():
    # BBB: We can obviously remove this when testbrowser is Plone
    #      mainstream, read: with Five 1.4.
    try:
        import Products.Five.testbrowser
    except ImportError:
        print >> sys.stderr, ("testbrowser not found; "
                              "testbrowser tests skipped")
        filenames = list_nontestbrowser_tests()
    else:
        filenames = list_doctests()

    suites = [
              Suite(os.path.basename(filename),
               optionflags=OPTIONFLAGS,
               package=PACKAGE,
               setUp=setUp,
               test_class=PloneTestCase.FunctionalTestCase,)
              for filename in filenames]

    # BBB: Fix for http://zope.org/Collectors/Zope/2178
    from Products.PloneTestCase import layer
    from Products.PloneTestCase import setup

    if setup.USELAYER:
        for s in suites:
            if not hasattr(s, 'layer'):
                s.layer = layer.PloneSite

    return unittest.TestSuite(suites)

if __name__ == '__main__':
    framework()




#!/usr/bin/python
#
"""
    This script reads awstats conf files and updates stats with all apache log files (archived too).
"""

import os
import datetime
import sys
import shutil

def verbose(*messages):
    print '>>', ' '.join(messages)
def error(*messages):
#    print >>sys.stderr, '!!', (' '.join(messages))
    print '!!', (' '.join(messages))
def debug(*messages):
    if not TRACE:
        return
    print 'TRACE:', ' '.join(messages)
#------------------------------------------------------------------------------

#CONF_DIR = '/etc/awstats'
CONF_DIR = '/home/srv/temp/awstats'
#APACHE_DIR = '/var/log/apache2'
APACHE_DIR = '/home/srv/temp/apache2_logs'
INCLUDE_ARCHIVED = False
AWSTATS_CMD = '/usr/lib/cgi-bin/awstats.pl'
TRACE = False
TEMP_DIR = '/tmp/'
gunzip = 'gunzip'

#------------------------------------------------------------------------------

def getFiles(dirpath, extensions, namestart, sort=False):
    """ Read the dir and return some files """
    files = []
    for filename in os.listdir(dirpath):
        debug("filename='%s'"%filename)
        if filename in ('.','..'):
            continue
        extension = os.path.splitext(filename)[1].strip('.')
        debug("extension='%s'"%extension)
        if not extension:
            continue
        if extensions and extension not in extensions:
            continue
        if namestart and not filename.startswith(namestart):
            continue
        debug("Kept '%s'"%filename)
        filepath = os.path.join(dirpath, filename)
        files.append(filepath)
    if sort:
        files.sort(compare_file_modiftime)
    return files

#-------------------------------------------------

def readFile(filename, lines):
    """ read the filename and return lines """
    try:
        zfile = open( filename, 'r')
    except IOError:
        error("! Cannot open %s file" % filename)
        return
#    lines = zfile.readlines()
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        lines.append(line)
    zfile.close()

#------------------------------------------------------------------------------

def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    ret = os.system(cmd + ' >_cmd_pv.out 2>_cmd_pv.err')
    stdout = stderr = []
    try:
        if os.path.exists('_cmd_pv.out'):
            ofile = open( '_cmd_pv.out', 'r')
            stdout = ofile.readlines()
            ofile.close()
            os.remove('_cmd_pv.out')
        else:
            error("File %s does not exist" % '_cmd_pv.out')
    except IOError:
        error("Cannot open %s file" % '_cmd_pv.out')
    try:
        if os.path.exists('_cmd_pv.err'):
            ifile = open( '_cmd_pv.err', 'r')
            stderr = ifile.readlines()
            ifile.close()
            os.remove('_cmd_pv.err')
        else:
            error("File %s does not exist" % '_cmd_pv.err')
    except IOError:
        error("Cannot open %s file" % '_cmd_pv.err')
    return( stdout,stderr )

#------------------------------------------------------------------------------

def compare_file_modiftime(file1, file2):
    """ compare file in function of modification time """
    date1 = datetime.datetime.fromtimestamp(os.stat(file1).st_mtime)
    date2 = datetime.datetime.fromtimestamp(os.stat(file2).st_mtime)
    return cmp(date1, date2)

#------------------------------------------------------------------------------

def main():
    verbose("Reading conf files")
    for conffilepath in getFiles(CONF_DIR, ('conf',), '', sort=True):
        conffile = os.path.basename(conffilepath)
        verbose("Reading %s"%conffile)
        lines = []
        readFile(conffilepath, lines)
        for line in lines:
            line = line.strip('\n\t ')
#            debug("line '%s'"%line)
            if line.startswith('LogFile'):
                line = line[7:].strip('=" ')
                #to avoid line like LogFile="/usr/share/doc/awstats/examples/logresolvemerge.pl /srv/apache/apache2_logs/*_access.log |"
                if line.find('|') >= 0:
                    break
                logfilename = os.path.basename(line)
                verbose("\tFound apache log name '%s'"%logfilename)
                logfiles = getFiles(APACHE_DIR, [], logfilename, sort=True)
                if not logfiles:
                    error("No log files found in '%s'"%APACHE_DIR)
                #we remove begin and end: awstats. xxx .conf 
                configname = conffile[8:-5]
                verbose("\tconfig name = '%s'"%configname)
                for logfilepath in logfiles:
                    logfile = os.path.basename(logfilepath)
                    if not (logfile.endswith('.log') or logfile.endswith('.log.1') or logfile.endswith('.gz')):
                        error("Logfile found but not recognized '%s'"%logfile)
                        continue
                    verbose("\tLogfile found '%s'"%logfile)
                    if logfile.endswith('.gz'):
                        #we need to decompress the file
                        destination = os.path.join(TEMP_DIR, logfile)
                        try:
                            shutil.copyfile(logfilepath, destination)
                            verbose( "\t'%s' copied to '%s'" % (logfilepath, destination))
                        except Exception, errmsg:
                            error( "'%s' NOT COPIED to '%s'" % (logfilepath, destination))
                            error(str(errmsg))
                        command = "%s -f %s"%(gunzip, destination)
                        (cmd_out, cmd_err) = runCommand(command)
                        if cmd_err:
                            error("error running command %s : %s" % (command, ''.join(cmd_err)))
                        logfilepath = destination[:-3]
                    command = "%s -config=%s -Logfile=%s -update"%(AWSTATS_CMD, configname, logfilepath)
                    verbose("\t>> Running '%s'"%command)
                    (cmd_out, cmd_err) = runCommand(command)
                    if cmd_err:
                        error("error running command %s : %s" % (command, ''.join(cmd_err)))
                    if cmd_out:
                        verbose("\t>>OUTPUT: %s" % (''.join(cmd_out)))
                    if logfile.endswith('.gz'):
                        os.remove(logfilepath)
                        verbose("\t'%s' deleted"%logfilepath)
                break
    verbose("End of script")

#------------------------------------------------------------------------------

try:
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-a", "--archived", dest="archived",
                  default=False,
                  help="Include archived logs or not. Default=False")
    (options, args) = parser.parse_args()
    INCLUDE_ARCHIVED = options.archived
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# File: tests.py
#
# Copyright (c) 2007 by CommunesPlone
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan GEULETTE <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'


##code-section init-module-header #fill in your manual code here
##/code-section init-module-header


# Subpackages
# Additional

# Classes

##code-section init-module-footer #fill in your manual code here
##/code-section init-module-footer




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# André NUYENS <andre.nuyens@uvcw.be>, UVCW
#

import os, sys
import urllib
from datetime import datetime, timedelta

TRACE = False

def verbose(*messages):
    print '>>', ' '.join(messages)
def error(*messages):
    print '!!', (' '.join(messages))
def trace(*messages):
    if not TRACE:
        return
    print 'TRACE:', ' '.join(messages)

#------------------------------------------------------------------------------

def read_zopectlfile(zopectlfilename):
    """ read the zopectl file to find the zope path """
    try:
        zfile = open( zopectlfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % zopectlfilename)
        return
    zopepath = ''
    pythonfile = ''
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('ZOPE_HOME'):
            zopepath = line.split('=')[1]
            zopepath = zopepath.strip('"\' ')
        elif line.startswith('PYTHON='):
            pythonfile = line.split('=')[1]
            pythonfile = pythonfile.strip('"\' ')
    return(zopepath, pythonfile)
    
#------------------------------------------------------------------------------

def read_zopeconffile(zodbfilename, lines):
    """ read the zope conf filename and include subfile """
    try:
        zfile = open( zodbfilename, 'r')
    except IOError:
        error("! Cannot open %s file" % zodbfilename)
        return
    for line in zfile.readlines():
        line = line.strip('\n\t ')
        if line.startswith('%include'):
            otherfilename = line.split()[1]
            read_zopeconffile(otherfilename, lines)
            continue
        lines.append(line)
    zfile.close()

#------------------------------------------------------------------------------

def treat_zopeconflines(zodbfilename, fspath):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)

    httpflag = False
    mp_name = mp_path = mp_fs = mp_fspath = None
    mp_size = 0
    for line in lines:
        line = line.strip()
        if line.startswith('<http-server>'):
            httpflag = True
            continue
        if line.startswith('<zodb_db '):
            #<zodb_db main>
            if mp_name:
                error("\tnext db found while end tag not found: previous dbname '%s', current line '%s'"%(mp_name, line))
            mp_name = line.split()[1]
            mp_name = mp_name.strip('> ')
            if mp_name == 'temporary':
                mp_name = None
            continue
        if mp_name and line.startswith('</zodb_db>'):
            mp_name = mp_path = mp_fs = mp_fspath = None
            mp_size = 0
            continue
        if mp_name and line.startswith('path '):
            #path $INSTANCE/var/Data.fs
            mp_fs = os.path.basename(line.split()[1])
            if not mp_fs.endswith('.fs'):
                error("Error getting fs name in '%s'"%line)
            mp_fspath = os.path.join(fspath, mp_fs)
            if os.path.exists(mp_fspath):
                mp_size = int(os.path.getsize(mp_fspath)/1048576)
            else:
                error("Db file '%s' doesn't exist"%mp_fspath)
                mp_fspath = ''
            continue
        if mp_name and line.startswith('mount-point '):
            #mount-point /
            mp_path = line.split()[1]
            if not mp_path.startswith('/'):
                error("Error getting path name in '%s'"%line)
            continue
        if httpflag and line.startswith('address'):
            port = line.split()[1]
    return port
    
#------------------------------------------------------------------------------

class MyUrlOpener(urllib.FancyURLopener):
    """ redefinition of this class to give the user and password"""
    def prompt_user_passwd(self, host, realm):
        return (user,pwd)
    def __init__(self, *args):
        self.version = "Zope Packer"
        urllib.FancyURLopener.__init__(self, *args)
 
 #------------------------------------------------------------------------------

def packdb(port, db, days, method, module, function, user_, pwd_):
    global user, pwd
    user = user_
    pwd = pwd_
    host = "http://localhost:%s" % port
    urllib._urlopener = MyUrlOpener()
    start = datetime.now()
    verbose("\tPacking db '%s' for instance %s (%s days)" % (db, host, days))    
    url_spd = "%s/Control_Panel/Database/%s/%s?days:float=%s" % (host, db, method, days)
    #verbose("url='%s'"%url_spd)
    try:
        ret_html = urllib.urlopen(url_spd).read()
        if 'the requested resource does not exist' in ret_html or 'error was encountered while publishing this resource' in ret_html:
            verbose('\texternal method %s not exist : we will create it'%method)
            url_em = "%s/manage_addProduct/ExternalMethod/manage_addExternalMethod?id=%s&module=%s&function=%s&title="%(host, method, module, function)
            try:
                ret_html = urllib.urlopen(url_em).read()
                if 'the requested resource does not exist' in ret_html or 'error was encountered while publishing this resource' in ret_html:
                    error('! Cannot create external method in zope')
                elif 'The specified module,' in ret_html:
                    error('! The specified module %s is not present in the instance'%module)
                else:
                    try:
                        ret_html = urllib.urlopen(url_spd).read()
#                            if "/Control_Panel/Database/%s"%db not in ret_html:
#                                error("Problem during compression of %s"%db)
#                                log.debug(ret_html) 
                        verbose("\t%s"%ret_html)
                    except IOError, msg:
                        error("! Cannot open URL %s, aborting : %s" % (url_spd, msg))
            except Exception, msg:
                error("! Cannot open URL %s, aborting : %s" % (url_em, msg))
        else:
            verbose("\t%s"%ret_html)
    except IOError, msg:
        error("! Cannot open URL %s, aborting : %s" % (url_spd, msg))
    verbose("\t\t-> elapsed time %s"%(datetime.now()-start))
           
#------------------------------------------------------------------------------

def CreateAndCallExternalMethod(port, user_, pwd_, ext_method, ext_filename,function,param=''):    
    #creating and calling external method in zope
    global instdir, tempdir, buildout_inst_type, user, pwd
    host = "http://localhost:%s" % port
    user = user_
    pwd = pwd_
    urllib._urlopener = MyUrlOpener()
    url_pv = "%s/%s%s" % (host, ext_method, param)
    current_url = url_pv
    try:
        #verbose("Running '%s'"%current_url)
        ret_html = urllib.urlopen(current_url).read()
        if 'the requested resource does not exist' in ret_html:
            verbose('external method %s not exist : we will create it'%ext_method)
            (module, extension) = os.path.splitext(ext_filename)
            module = 'CPUtils.' + module
            current_url = "%s/manage_addProduct/ExternalMethod/manage_addExternalMethod?id=%s&module=%s&function=%s&title="%(host, ext_method, module, function)
            #verbose("Running now '%s'"%current_url)
            ret_html = urllib.urlopen(current_url).read()  
            if 'the requested resource does not exist' in ret_html or \
                ('The specified module' in ret_html and "couldn't be found" in ret_html):
                error("Cannot create external method in zope : '%s'"%ret_html)
                sys.exit(1)
            else:
                current_url = "%s/%s/valid_roles" % (host, ext_method)
                #verbose("Running now '%s'"%current_url)
                ret_html = urllib.urlopen(current_url).read()
                if not ret_html[0] == '(':
                    error("error with valid_roles return: '%s'"%ret_html)
                    sys.exit(1)
                valid_roles = list(eval(ret_html))
                managerindex = valid_roles.index('Manager')
                current_url = "%s/%s/permission_settings" % (host, ext_method)
                #verbose("Running now '%s'"%current_url)
                ret_html = urllib.urlopen(current_url).read()
                if not ret_html[0] == '[':
                    error("error with permission_settings return: '%s'"%ret_html)
                    sys.exit(1)
                permission_settings = eval(ret_html)   
                params = {}             
                count = 0
                for perm in permission_settings:
                    if perm['name'] in ('Access contents information','View'):
                        params['p%dr%d'%(count,managerindex)] = 'on'
                    else:
                        params['a%d'%count] = 'on'
                    count += 1
                current_url = "%s/%s/manage_changePermissions" % (host, ext_method)
                #verbose("Running now '%s'"%current_url)
# params example                       params = {  'permission_to_manage':'View', 
#                                    'roles':['Manager'], }
                data = urllib.urlencode(params)
                ret_html = urllib.urlopen(current_url, data).read()
                if 'Your changes have been saved' not in ret_html:
                    error("Error changing permissions with URL '%s', data '%s'" % (current_url,str(data)))
                    sys.exit(1)
                current_url = url_pv
                #verbose("Running again '%s'"%current_url)
                ret_html = urllib.urlopen(current_url).read()
        #verbose("callAndCreateExternalMethod : \n%s"%ret_html)
        if ret_html:
            verbose('%s'%ret_html)
    except Exception, msg:
        error("Cannot open URL %s, aborting: '%s'" % (current_url,msg))
        sys.exit(1)
